'use client';

import { z } from 'zod';
import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import Link from 'next/link';
import { signIn, signUp } from '@/lib/queries';
import { useRouter } from 'next/navigation';
import { useEffect, useState } from 'react';
import { useAuth } from '@/lib/hooks/context';
import { useSupabaseClient } from '@/utils/supabase';
import { StatusMessage, SubmitButton } from '@/components/default/forms';
import {
  Card,
  CardContent,
  CardDescription,
  CardHeader,
  CardTitle,
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
  Input,
  Separator,
  Tabs,
  TabsContent,
  TabsList,
  TabsTrigger,
} from '@/components/ui';
import {
  SignInWithApple,
  SignInWithMicrosoft
} from '@/components/default/auth/buttons/client';

const signInFormSchema = z.object({
  email: z.string().email({
    message: 'Please enter a valid email address.'
  }),
  password: z.string().min(8, {
    message: 'Password must be at least 8 characters.'
  }),
});

const signUpFormSchema = z
  .object({
    name: z.string().min(2, {
      message: 'Name must be at least 2 characters.',
    }),
    email: z.string().email({
      message: 'Please enter a valid email address.',
    }),
    password: z.string().min(8, {
      message: 'Password must be at least 8 characters.',
    }),
    confirmPassword: z.string().min(8, {
      message: 'Password must be at least 8 characters.',
    }),
  })
  .refine((data) => data.password === data.confirmPassword, {
    message: 'Passwords do not match!',
    path: ['confirmPassword'],
  });

export const SignInCard = () => {
  const router = useRouter();
  const { isAuthenticated, loading, refreshUser } = useAuth();
  const [statusMessage, setStatusMessage] = useState('');
  const supabase = useSupabaseClient();

  const signInForm = useForm<z.infer<typeof signInFormSchema>>({
    resolver: zodResolver(signInFormSchema),
    defaultValues: {
      email: '',
      password: '',
    },
  });

  const signUpForm = useForm<z.infer<typeof signUpFormSchema>>({
    resolver: zodResolver(signUpFormSchema),
    defaultValues: {
      name: '',
      email: '',
      password: '',
      confirmPassword: '',
    },
  });

  useEffect(() => {
    if (isAuthenticated) router.push('/');
  }, [isAuthenticated, router]);

  const handleSignIn = async (values: z.infer<typeof signInFormSchema>) => {
    try {
      setStatusMessage('');
      const formData = new FormData();
      formData.append('email', values.email);
      formData.append('password', values.password);
      if (!supabase) throw new Error('Supabase client not found');
      const result = await signIn(supabase, formData);
      if (result.error) throw new Error(result.error.message);
      await refreshUser();
      signInForm.reset();
      router.push('');
    } catch (error) {
      setStatusMessage(`Error signing in: ${error as string}`);
    }
  };

  const handleSignUp = async (values: z.infer<typeof signUpFormSchema>) => {
    try {
      setStatusMessage('');
      const formData = new FormData();
      formData.append('name', values.name);
      formData.append('email', values.email);
      formData.append('password', values.password);
      if (!supabase) throw new Error('Supabase client not found');
      const result = await signUp(supabase, formData);
      if (result.error) throw new Error(result.error.message);
      await refreshUser();
      signUpForm.reset();
      router.push('');
    } catch (error) {
      setStatusMessage(`Error signing up: ${error as string}`);
    }
  };

  return (
    <Tabs defaultValue='sign-in'>
      <TabsList>
        <TabsTrigger value='sign-in'>Sign In</TabsTrigger>
        <TabsTrigger value='sign-up'>Sign Up</TabsTrigger>
      </TabsList>
      <TabsContent value='sign-in'>
        <Card className='min-w-xs md:min-w-sm max-w-lg'>
          <CardHeader>
            <CardTitle className=''>
              Sign In
            </CardTitle>
          </CardHeader>
          <CardContent>
            <Form {...signInForm}>
              <form
                onSubmit={signInForm.handleSubmit(handleSignIn)}
                className=''
              >
                <FormField
                  control={signInForm.control}
                  name='email'
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel className=''>Email</FormLabel>
                      <FormControl>
                        <Input
                          type='email'
                          placeholder='you@example.com'
                          {...field}
                        />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  control={signInForm.control}
                  name='password'
                  render={({ field }) => (
                    <FormItem>
                      <div className='flex justify-between'>
                        <FormLabel className=''>Password</FormLabel>
                        <Link
                          href='/forgot-password'
                        >
                          Forgot Password?
                        </Link>
                      </div>
                      <FormControl>
                        <Input
                          type='password'
                          placeholder='Your password'
                          {...field}
                        />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                {statusMessage &&
                  (statusMessage.includes('Error') ||
                  statusMessage.includes('error') ||
                  statusMessage.includes('failed') ||
                  statusMessage.includes('invalid') ? (
                    <StatusMessage message={{ error: statusMessage }} />
                  ) : (
                    <StatusMessage message={{ message: statusMessage }} />
                ))}
                <SubmitButton
                  disabled={loading}
                  pendingText='Signing In...'
                >
                  Sign In
                </SubmitButton>
              </form>
            </Form>
            <div className='flex items-center w-full gap-4'>
              <Separator className='flex-1 bg-accent py-0.5' />
              <span className='text-sm text-muted-foreground'>or</span>
              <Separator className='flex-1 bg-accent py-0.5' />
            </div>
            <SignInWithMicrosoft />
            <SignInWithApple />
          </CardContent>
        </Card>
      </TabsContent>
      <TabsContent value='sign-up'>
        <Card className='min-w-xs md:min-w-sm max-w-lg'>
          <CardHeader>
            <CardTitle className=''>
              Sign Up
            </CardTitle>
          </CardHeader>
          <CardContent>
            <Form {...signUpForm}>
              <form
                onSubmit={signUpForm.handleSubmit(handleSignUp)}
                className=''
              >
                <FormField
                  control={signUpForm.control}
                  name='name'
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel className=''>Name</FormLabel>
                      <FormControl>
                        <Input
                          type='text'
                          placeholder='Full Name'
                          {...field}
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />
                <FormField
                  control={signUpForm.control}
                  name='email'
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel className=''>Email</FormLabel>
                      <FormControl>
                        <Input
                          type='email'
                          placeholder='you@example.com'
                          {...field}
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />
                <FormField
                  control={signUpForm.control}
                  name='password'
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel className=''>Password</FormLabel>
                      <FormControl>
                        <Input
                          type='password'
                          placeholder='Your password'
                          {...field}
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />
                <FormField
                  control={signUpForm.control}
                  name='confirmPassword'
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel className=''>Confirm Passsword</FormLabel>
                      <FormControl>
                        <Input
                          type='password'
                          placeholder='Confirm your password'
                          {...field}
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />
                {statusMessage &&
                  (statusMessage.includes('Error') ||
                  statusMessage.includes('error') ||
                  statusMessage.includes('failed') ||
                  statusMessage.includes('invalid') ? (
                    <StatusMessage message={{ error: statusMessage }} />
                  ) : (
                    <StatusMessage message={{ success: statusMessage }} />
                ))}
                <SubmitButton
                  disabled={loading}
                  pendingText='Signing Up...'
                >
                  Sign Up
                </SubmitButton>
              </form>
            </Form>
            <div className='flex items-center w-full gap-4'>
              <Separator className='flex-1 bg-accent py-0.5' />
              <span className='text-sm text-muted-foreground'>or</span>
              <Separator className='flex-1 bg-accent py-0.5' />
            </div>
            <SignInWithMicrosoft />
            <SignInWithApple />
          </CardContent>
        </Card>
      </TabsContent>
    </Tabs>
  );

};

