'use client';
import {
  QueryClient,
  QueryClientProvider as ReactQueryClientProvider,
  QueryCache,
  MutationCache,
} from '@tanstack/react-query';
import { useState } from 'react';
import { toast } from 'sonner';

const enum QueryErrorCodes {
  FETCH_USER_FAILED = 'FETCH_USER_FAILED',
  FETCH_PROFILE_FAILED = 'FETCH_PROFILE_FAILED',
  FETCH_AVATAR_FAILED = 'FETCH_AVATAR_FAILED',
  UPDATE_PROFILE_FAILED = 'UPDATE_PROFILE_FAILED',
  UPLOAD_PHOTO_FAILED = 'UPLOAD_PHOTO_FAILED',
};

const queryCacheOnError = (error: unknown, query: any) => {
  const errorMessage = error instanceof Error ? error.message : error as string;
  switch (query.meta?.errCode) {
    case QueryErrorCodes.FETCH_USER_FAILED:
      toast.error('Failed to fetch user!');
      break;
    case QueryErrorCodes.FETCH_PROFILE_FAILED:
      toast.error('Failed to fetch profile!');
      break;
    case QueryErrorCodes.FETCH_AVATAR_FAILED:
      console.warn('Failed to fetch avatar. User may not have one!')
      break;
    default:
      console.error('Query error:', error);
      break;
  }
};

const mutationCacheOnError = (
  error: unknown,
  variables: unknown,
  context: unknown,
  mutation: any,
) => {
  const errorMessage = error instanceof Error ? error.message : error as string;
  switch (mutation.meta?.errCode) {
    case QueryErrorCodes.UPDATE_PROFILE_FAILED:
      toast.error(`Failed to update user profile: ${errorMessage}`)
      break;
    case QueryErrorCodes.UPLOAD_PHOTO_FAILED:
      toast.error(`Failed to upload photo: ${errorMessage}`)
      break;
    default:
      console.error('Mutation error:', error);
      break;
  }
};


const QueryClientProvider = ({ children }: { children: React.ReactNode }) => {
  const [queryClient] = useState(
    () =>
      new QueryClient({
        queryCache: new QueryCache({
          onError: queryCacheOnError,
        }),
        mutationCache: new MutationCache({
          onError: mutationCacheOnError,
        }),
        defaultOptions: {
          queries: {
            staleTime: 60 * 1000,
          },
        },
      })
  )
  return <ReactQueryClientProvider client={queryClient}>{children}</ReactQueryClientProvider>
};

export { QueryClientProvider, QueryErrorCodes };
